package com.link.core.basic.es.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.link.base.datatask.model.DataTask;
import com.link.base.user.model.CoreUser;
import com.link.core.basic.annotations.JsonParam;
import com.link.core.basic.controller.BasicControllerException;
import com.link.core.basic.es.service.BasicEsService;
import com.link.core.basic.model.BasicModel;
import com.link.core.basic.query.QueryParams;
import com.link.core.basic.service.BasicServiceException;
import com.link.core.basic.service.KeyGenerateService;
import com.link.core.basic.thread.LinkThreadPoolExecutor;
import com.link.core.cllog.LCLogger;
import com.link.core.modules.export.ExportFactory;
import com.link.core.modules.export.ExportWriter;
import com.link.core.util.DateUtil;
import com.link.core.util.PropertyUtil;
import com.link.core.util.RedisUtil;
import com.link.core.util.SpringUtil;
import com.link.core.util.StringUtils;
import com.link.core.util.UserUtil;
import com.link.core.util.WebUtils;
import com.link.core.util.asserts.AssertsUtis;
import com.link.microservice.DataTaskBaseClient;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

@Controller
@RequestMapping({"/link/es"})
public class BasicEsController<T extends BasicModel> {
   private static final Logger logger = LogManager.getLogger(BasicEsController.class);
   @Resource(
      name = "LinkThreadPoolExecutorEsExport"
   )
   private LinkThreadPoolExecutor LinkThreadPoolExecutorEsExport;
   @Resource
   private KeyGenerateService keyGenerateService;
   @Resource
   private DataTaskBaseClient dataTaskBaseClient;

   public BasicEsService<T> getBasicService() throws Exception {
      throw new BasicControllerException("获取Basic Service 失败，请确认程序代码是否正确！");
   }

   @RequestMapping({"/queryByExamplePage"})
   @ResponseBody
   public Map<String, Object> queryByExamplePage(@JsonParam QueryParams qps, @JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      HashMap<String, Object> result = new HashMap(8);

      try {
         qps.preDealSecurity(request);
         T temp = BasicModel.transformClass(entity, qps);
         this.beforeQueryAllOrExample(temp, request);
         this.beforeQueryExample(temp, request);
         List<T> list = this.getBasicService().queryByExamplePage(temp);
         result.put("success", true);
         result.put("rows", list);
         result.put("total", getCount(temp));
         result.put("code", "200");
      } catch (BasicServiceException var9) {
         BasicServiceException var9 = var9;
         logger.error("BasicEsController捕获异常: ", var9);
         result.put("success", false);
         result.put("result", var9.getMessage());
         result.put("detailMessage", var9.getDetailMessage());
         result.put("code", var9.getCode());
      } catch (Exception var10) {
         Exception var10 = var10;
         logger.error("BasicEsController捕获异常: ", var10);
         result.put("success", false);
         result.put("result", var10.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   public static Integer getCount(BasicModel basicModel) {
      int total = basicModel.getTotal();
      return basicModel.getPageFlag() && !basicModel.getTotalFlag() ? basicModel.getStartRow() + total - 1 : total;
   }

   @RequestMapping({"/queryByExamplePage/exportData"})
   @ResponseBody
   public Map<String, Object> exportData(@JsonParam final T entity, @JsonParam final QueryParams qps, final HttpServletRequest request, HttpServletResponse response) {
      logger.info("ES导出列表...");
      Map<String, Object> result = new HashMap(4);

      try {
         String asyncExportFlag = qps.getAsyncExportFlag();
         final String exportTaskType = qps.getExportTaskType();
         final String exportCurrentPage = qps.getExportCurrentPage();
         qps.setEsExportFlag(true);
         if ("Y".equals(asyncExportFlag)) {
            logger.info("异步导出ES列表...");
            final DataTask dataTask = this.generateDataTask((DataTask)null, "new", exportTaskType, "");
            this.saveDataTaskParams(entity, qps, dataTask);
            String serverName = ((Environment)SpringUtil.getBean(Environment.class)).getProperty("spring.application.name");
            String exportThreads = PropertyUtil.getCommonProperty("ES_EXPORT_TRHEAD_NUM");
            if (StringUtils.isNotBlank(exportThreads)) {
               JSONObject jsonObject = JSONObject.parseObject(exportThreads);
               if (jsonObject.containsKey(serverName)) {
                  Integer threadNum = jsonObject.getInteger(serverName);
                  this.LinkThreadPoolExecutorEsExport.setCorePoolSize(threadNum);
               } else {
                  this.LinkThreadPoolExecutorEsExport.setCorePoolSize(20);
               }
            }

            this.LinkThreadPoolExecutorEsExport.execute(new Runnable() {
               public void run() {
                  BasicEsController.this.asyncExport(entity, qps, exportTaskType, exportCurrentPage, request, dataTask);
               }
            });
         } else {
            logger.info("同步导出列表..." + qps.getExportFileName());
            String urlPath = "";
            if ("Y".equals(exportCurrentPage)) {
               this.beforExportExcel(entity, qps, request);
               urlPath = this.exportDataHandler(entity, qps);
            } else {
               this.beforExportExcelCheck(entity, qps, request);
               this.beforExportExcel(entity, qps, request);
               urlPath = this.scrollExportDataHandler(entity, qps);
            }

            result.put("url", urlPath);
         }

         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var14) {
         BasicServiceException e = var14;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var15) {
         Exception e = var15;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   private DataTask generateDataTask(DataTask dataTask, String type, String exportTaskType, String urlPath) throws Exception {
      if (dataTask == null) {
         dataTask = new DataTask();
         dataTask.setId(this.keyGenerateService.keyGenerate());
      }

      dataTask.setImportType(exportTaskType);
      dataTask.setType("Export");
      switch (type) {
         case "new":
            dataTask.setTaskStatus("Running");
            dataTask.setTaskMessage("正在导出");
            dataTask.setStartTime(DateUtil.dateToStr(new Date(), "yyyy-MM-dd HH:mm:ss"));
            this.dataTaskBaseClient.insert(JSONObject.parseObject(JSON.toJSONString(dataTask)));
            break;
         case "success":
            dataTask.setTaskStatus("Completed");
            dataTask.setTaskMessage("已完成");
            dataTask.setEndTime(DateUtil.dateToStr(new Date(), "yyyy-MM-dd HH:mm:ss"));
            dataTask.setFeedbackPath(urlPath);
            this.dataTaskBaseClient.update(JSONObject.parseObject(JSON.toJSONString(dataTask)));
            break;
         case "error":
            dataTask.setTaskStatus("Error");
            dataTask.setTaskMessage("导出失败" + urlPath);
            dataTask.setEndTime(DateUtil.dateToStr(new Date(), "yyyy-MM-dd HH:mm:ss"));
            this.dataTaskBaseClient.update(JSONObject.parseObject(JSON.toJSONString(dataTask)));
      }

      return dataTask;
   }

   private DataTask saveDataTaskParams(T entity, QueryParams qps, DataTask dataTask) throws Exception {
      AssertsUtis.isBlank(dataTask.getId(), "未生成导出");
      String entityStr = JSONObject.toJSONString(entity);
      String qpsStr = JSONObject.toJSONString(qps);
      dataTask.setFilterParm1(entityStr);
      dataTask.setFilterParm2(qpsStr);
      if (StringUtils.isBlank(dataTask.getFilterParm3())) {
         dataTask.setFilterParm3("1");
      }

      dataTask.setUpdateFields("filterParm1,filterParm2,filterParm3");
      this.dataTaskBaseClient.update(JSONObject.parseObject(JSON.toJSONString(dataTask)));
      dataTask.setUpdateFields((String)null);
      return dataTask;
   }

   private void retryExport(DataTask dataTask, String exportTaskType, Exception e, HttpServletRequest request, Class<?> cla) throws Exception {
      Map<String, Object> dataMap = this.dataTaskBaseClient.queryById(JSONObject.parseObject(JSON.toJSONString(dataTask)));
      Boolean successFlg = (Boolean)dataMap.get("success");
      if (!successFlg) {
         throw new BasicServiceException(dataMap.get("result").toString());
      } else {
         DataTask tempDataTask = (DataTask)JSON.parseObject(JSON.toJSONString(dataMap.get("result")), DataTask.class);
         if (StringUtils.isNotBlank(tempDataTask.getFilterParm3()) && Integer.parseInt(tempDataTask.getFilterParm3()) < 3) {
            tempDataTask.setFilterParm3(Integer.parseInt(tempDataTask.getFilterParm3()) + 1 + "");
            tempDataTask.setUpdateFields("filterParm3");
            this.dataTaskBaseClient.update(JSONObject.parseObject(JSON.toJSONString(tempDataTask)));
            tempDataTask.setUpdateFields((String)null);
            String entityStr = tempDataTask.getFilterParm1();
            String qpsStr = tempDataTask.getFilterParm2();
            T entity = (BasicModel)JSONObject.parseObject(entityStr, cla);
            QueryParams qps = (QueryParams)JSONObject.parseObject(qpsStr, QueryParams.class);
            this.asyncExport(entity, qps, qps.getExportTaskType(), qps.getExportCurrentPage(), request, tempDataTask);
         } else {
            this.generateDataTask(dataTask, "error", exportTaskType, e.getMessage());
         }

      }
   }

   private void asyncExport(T entity, QueryParams qps, String exportTaskType, String exportCurrentPage, HttpServletRequest request, DataTask dataTask) {
      try {
         String urlPath = "";
         logger.info("异步es导出开始日志标识:" + qps.getExportFileName());
         if ("Y".equals(exportCurrentPage)) {
            this.beforExportExcel(entity, qps, request);
            urlPath = this.exportDataHandler(entity, qps);
         } else {
            this.beforExportExcelCheck(entity, qps, request);
            this.beforExportExcel(entity, qps, request);
            urlPath = this.scrollExportDataHandler(entity, qps);
         }

         this.generateDataTask(dataTask, "success", exportTaskType, urlPath);
         logger.info("异步es导出结束日志标识:" + qps.getExportFileName());
      } catch (Exception var10) {
         Exception e = var10;
         LCLogger.withException(e);

         try {
            this.retryExport(dataTask, exportTaskType, e, request, entity.getClass());
            logger.info("异步es导出异常日志标识:" + qps.getExportFileName());
         } catch (Exception var9) {
            Exception e1 = var9;
            LCLogger.withException("导出任务创建失败：" + e1);
         }
      }

   }

   private String exportDataHandler(T entity, QueryParams qps) throws Exception {
      CoreUser user = UserUtil.getUser();
      if (null == user) {
         throw new BasicServiceException("未查询到当前登录用户信息，导出失败");
      } else {
         user.setExportType(qps.getExportFileType());
         ExportWriter exportWriter;
         if ("xlsx".equals(user.getExportType())) {
            exportWriter = ExportFactory.newExcelInstance(qps);
         } else {
            exportWriter = ExportFactory.newInstance(qps);
         }

         T temp = BasicModel.transformClass(entity, qps);
         this.beforeQueryAllOrExample(temp, (HttpServletRequest)null);
         this.beforeQueryExample(temp, (HttpServletRequest)null);

         while(!qps.getStopExport()) {
            List<T> list = this.getBasicService().queryByExamplePage(temp);
            if (qps.getPage() == 1) {
               qps.setTotal(temp.getTotal());
            }

            exportWriter.wirteData(list);
            if ("Y".equals(qps.getExportCurrentPage())) {
               break;
            }

            temp.setPage(qps.getPage());
            temp.setTotalFlag(qps.getTotalFlag());
         }

         return exportWriter.getUrlPath();
      }
   }

   public String scrollExportDataHandler(T entity, QueryParams qps) throws Exception {
      CoreUser user = UserUtil.getUser();
      if (null == user) {
         throw new BasicServiceException("未查询到当前登录用户信息，导出失败");
      } else {
         user.setExportType(qps.getExportFileType());
         ExportWriter exportWriter;
         if ("xlsx".equals(user.getExportType())) {
            exportWriter = ExportFactory.newExcelInstance(qps);
         } else {
            exportWriter = ExportFactory.newInstance(qps);
         }

         T temp = BasicModel.transformClass(entity, qps);
         this.beforeQueryAllOrExample(temp, (HttpServletRequest)null);
         this.beforeScrollAll(temp, (HttpServletRequest)null);
         List<T> list = this.getBasicService().scrollQueryByExamplePage(temp);
         int exportNum = temp.getCurrentCount();
         if (!CollectionUtils.isEmpty(list) && !StringUtils.isBlank(((BasicModel)list.get(0)).getEsScrollId())) {
            int exportDataNum = 0;
            int resultSize = list.size();
            String scrollId = ((BasicModel)list.get(0)).getEsScrollId();
            temp.setEsScrollId(scrollId);
            exportWriter.writeScrollData(list);
            exportDataNum += resultSize;

            while(resultSize > 0 && !qps.getStopExport()) {
               list = this.getBasicService().scrollQueryByExamplePage(temp);
               if (CollectionUtils.isEmpty(list)) {
                  exportWriter.writeScrollData(new ArrayList());
                  break;
               }

               resultSize = list.size();
               exportWriter.writeScrollData(list);
               exportDataNum += resultSize;
               temp.setPage(qps.getPage());
            }

            Boolean isCheckExportException = "Y".equals(PropertyUtil.getCommonProperty("IS_CHECK_EXPORT_EXCEPTION"));
            AssertsUtis.isTrue(isCheckExportException && exportDataNum <= exportNum - 1000, "当前导出数据异常，麻烦稍后尝试重新导出");
            logger.info("导出数据总数为：" + exportNum + ",导出真实数据为：" + exportDataNum);
            return exportWriter.getUrlPath();
         } else {
            ExportWriter exportWriterEmpty;
            if ("xlsx".equals(user.getExportType())) {
               exportWriterEmpty = ExportFactory.newExcelInstance(qps);
            } else {
               exportWriterEmpty = ExportFactory.newInstance(qps);
            }

            logger.info("es查询进来没有...");
            exportWriterEmpty.emptyFile();
            return exportWriterEmpty.getUrlPath();
         }
      }
   }

   public void beforeQueryAllOrExample(T entity, HttpServletRequest request) throws Exception {
   }

   public void beforeQueryExample(T entity, HttpServletRequest request) throws Exception {
   }

   public void beforeScrollAll(T entity, HttpServletRequest request) throws Exception {
   }

   public void beforExportExcel(T entity, QueryParams qps, HttpServletRequest request) throws Exception {
   }

   public void beforExportExcelCheck(T entity, QueryParams queryParams, HttpServletRequest request) throws Exception {
      T t = (BasicModel)WebUtils.deepCopy(entity);
      QueryParams qps = (QueryParams)WebUtils.deepCopy(queryParams);
      qps.setPageFlag(true);
      T temp = BasicModel.transformClass(t, qps);
      temp.setOnlyCountFlag(true);
      this.beforeQueryExample(temp, request);
      this.beforeQueryAllOrExample(temp, (HttpServletRequest)null);
      this.beforeScrollAll(temp, (HttpServletRequest)null);
      List<T> list = this.getBasicService().queryByExamplePage(temp);
      if (CollectionUtils.isEmpty(list)) {
         entity.setCurrentCount(0);
      } else {
         int total = ((BasicModel)list.get(0)).getTotal();
         String data = ((BasicModel)list.get(0)).getClass().getSimpleName();
         logger.info("es当前导出对象-" + data);
         String exportData = RedisUtil.getProperty("exportEntityLimit", "0");
         if (StringUtils.isBlank(exportData)) {
            throw new BasicServiceException("企业参数配置参数为空exportEntityLimit");
         } else if (!exportData.contains(data) && total > 1000000) {
            throw new BasicServiceException("导出数据超出最大导出数据量1000000");
         } else {
            String exportLImitNumStr = RedisUtil.getCropProperty("EXPORT_LIMIT_NUM", "0");
            int exportLImitNum = StringUtils.isBlank(exportLImitNumStr) ? 1000000 : Integer.parseInt(exportLImitNumStr);
            DecimalFormat df = new DecimalFormat("#.##");
            BigDecimal sourceBd = (new BigDecimal(total)).divide(new BigDecimal(10000)).setScale(2, 4);
            BigDecimal errorBd = (new BigDecimal(exportLImitNum)).divide(new BigDecimal(10000)).setScale(2, 4);
            AssertsUtis.isTrue(total > exportLImitNum, ":待导出数据量" + df.format(sourceBd) + "万超过" + df.format(errorBd) + "万限制，请增加筛选条件");
            entity.setCurrentCount(total);
         }
      }
   }

   @RequestMapping({"/insert"})
   @ResponseBody
   public Map<String, Object> insert(@JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         this.beforInsert(entity, request);
         this.beforUpsert(entity, request);
         this.getBasicService().insert(entity);
         T newEntry = this.getBasicService().queryById(entity);
         result.put("newRow", newEntry);
         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var7) {
         BasicServiceException e = var7;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var8) {
         Exception e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping(
      value = {"/preDefaultValue"},
      method = {RequestMethod.POST},
      produces = {"application/json"}
   )
   @ResponseBody
   public Map<String, Object> preDefaultValue(T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         result.put("success", true);
         result.put("result", this.getBasicService().preDefaultValue(entity));
         result.put("code", "200");
      } catch (BasicServiceException var7) {
         BasicServiceException e = var7;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var8) {
         Exception e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/batchInsert"})
   @ResponseBody
   public Map<String, Object> batchInsert(@RequestBody List<T> list, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         for(int i = 0; list != null && i < list.size(); ++i) {
            T entity = (BasicModel)list.get(i);
            this.beforInsert(entity, request);
            this.beforUpsert(entity, request);
         }

         this.getBasicService().batchInsert(list);
         result.put("success", true);
         result.put("result", list);
         result.put("code", "200");
      } catch (BasicServiceException var8) {
         logger.error("BasicController捕获异常: ", var8);
         result.put("success", false);
         result.put("result", var8.getMessage());
         result.put("detailMessage", var8.getDetailMessage());
         result.put("code", var8.getCode());
      } catch (Exception var9) {
         Exception e = var9;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/batchUpdate"})
   @ResponseBody
   public Map<String, Object> batchUpdate(@RequestBody List<T> list, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         for(int i = 0; list != null && i < list.size(); ++i) {
            T entity = (BasicModel)list.get(i);
            this.beforUpdate(entity, request);
            this.beforUpsert(entity, request);
         }

         this.getBasicService().batchUpdate(list);
         result.put("success", true);
         result.put("result", list);
         result.put("code", "200");
      } catch (BasicServiceException var8) {
         logger.error("BasicController捕获异常: ", var8);
         result.put("success", false);
         result.put("result", var8.getMessage());
         result.put("detailMessage", var8.getDetailMessage());
         result.put("code", var8.getCode());
      } catch (Exception var9) {
         Exception e = var9;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/batchUpsert"})
   @ResponseBody
   public Map<String, Object> batchUpsert(@RequestBody List<T> list, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         for(int i = 0; list != null && i < list.size(); ++i) {
            T entity = (BasicModel)list.get(i);
            this.beforUpsert(entity, request);
         }

         this.getBasicService().batchUpsert(list);
         result.put("success", true);
         result.put("result", list);
         result.put("code", "200");
      } catch (BasicServiceException var8) {
         logger.error("BasicController捕获异常: ", var8);
         result.put("success", false);
         result.put("result", var8.getMessage());
         result.put("detailMessage", var8.getDetailMessage());
         result.put("code", var8.getCode());
      } catch (Exception var9) {
         Exception e = var9;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/queryById"})
   @ResponseBody
   public Map<String, Object> queryById(@JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         T record = this.getBasicService().queryById(entity);
         List<T> list = new ArrayList();
         list.add(record);
         this.pushCurrentDataCache(list, session.getId(), "base", false);
         result.put("result", record);
         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var8) {
         BasicServiceException e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var9) {
         Exception e = var9;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/upsert"})
   @ResponseBody
   public Map<String, Object> upsert(@JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         this.beforUpsert(entity, request);
         this.checkData(entity, request);
         if (!this.getBasicService().isInsertFlag(entity) && !this.checkExistDataCache("base", session, entity)) {
            throw new BasicServiceException("非法更新数据！");
         }

         this.getBasicService().upsert(entity);
         T newEntry = this.getBasicService().queryById(entity);
         result.put("newRow", newEntry);
         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var7) {
         BasicServiceException e = var7;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var8) {
         Exception e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/update"})
   @ResponseBody
   public Map<String, Object> update(@JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      Map<String, Object> result = new HashMap(4);

      try {
         this.beforUpdate(entity, request);
         this.beforUpsert(entity, request);
         this.checkData(entity, request);
         if (!this.checkExistDataCache("base", session, entity)) {
            throw new BasicServiceException("非法更新数据！");
         }

         this.getBasicService().update(entity);
         T newEntry = this.getBasicService().queryById(entity);
         result.put("newRow", newEntry);
         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var7) {
         BasicServiceException e = var7;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var8) {
         Exception e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   @RequestMapping({"/deleteById"})
   @ResponseBody
   public Map<String, Object> deleteById(@JsonParam T entity, HttpSession session, HttpServletRequest request, HttpServletResponse response) {
      logger.info("删除一条列表信息...");
      Map<String, Object> result = new HashMap(4);

      try {
         this.beforDelete(entity, request);
         this.checkData(entity, request);
         if (!this.checkExistDataCache("base", session, entity)) {
            throw new BasicServiceException("非法删除数据！");
         }

         this.getBasicService().deleteById(entity);
         result.put("success", true);
         result.put("code", "200");
      } catch (BasicServiceException var7) {
         BasicServiceException e = var7;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("detailMessage", e.getDetailMessage());
         result.put("code", e.getCode());
      } catch (Exception var8) {
         Exception e = var8;
         logger.error("BasicController捕获异常: ", e);
         result.put("success", false);
         result.put("result", e.getMessage());
         result.put("code", "400");
      }

      return result;
   }

   public void beforUpdate(T entity, HttpServletRequest request) throws Exception {
   }

   public void beforInsert(T entity, HttpServletRequest request) throws Exception {
   }

   public void beforDelete(T entity, HttpServletRequest request) throws Exception {
   }

   public void beforUpsert(T entity, HttpServletRequest request) throws Exception {
   }

   public void checkData(T entity, HttpServletRequest request) throws Exception {
   }

   public boolean checkExistDataCache(String key, HttpSession session, T entity) {
      BasicModel basicModel = null;
      if (entity instanceof BasicModel) {
         basicModel = entity;
      }

      if (basicModel == null) {
         throw new BasicServiceException("参数格式错误，请检查程序！");
      } else {
         String sessionId = session.getId();
         String basicModelId = basicModel.getId();
         return checkExistDataCache(key, sessionId, basicModelId);
      }
   }

   public static boolean checkExistDataCache(String key, String sessionId, String id) {
      return true;
   }

   public void pushCurrentDataCache(List<T> list, String sessionId, String key, boolean flag) {
   }
}
